@@ -1,12 +1,18 @@
+from __future__ import annotations
+
+import asyncio
 from os import environ
 from pathlib import Path
-from typing import TYPE_CHECKING, Dict, Optional, Sequence, Type
+from typing import TYPE_CHECKING
 
-from ..exceptions import PoeException
+from ..exceptions import ExecutionError
 from .base import PoeExecutor
 
 if TYPE_CHECKING:
-    from ..context import RunContext
+    from asyncio.subprocess import Process
+    from collections.abc import Sequence
+
+    from ..context import ContextProtocol
 
 
 class PoetryExecutor(PoeExecutor):
@@ -16,29 +22,29 @@ class PoetryExecutor(PoeExecutor):
     """
 
     __key__ = "poetry"
-    __options__: Dict[str, Type] = {}
 
     @classmethod
-    def works_with_context(cls, context: "RunContext") -> bool:
-        if "poetry" not in context.config.project["tool"]:
+    def works_with_context(cls, context: ContextProtocol) -> bool:
+        if not context.config.is_poetry_project:
             return False
         return bool(cls._poetry_cmd_from_path())
 
-    def execute(
-        self, cmd: Sequence[str], input: Optional[bytes] = None, use_exec: bool = False
-    ) -> int:
+    async def execute(
+        self, cmd: Sequence[str], input: bytes | None = None, use_exec: bool = False
+    ) -> Process:
         """
         Execute the given cmd as a subprocess inside the poetry managed dev environment
         """
 
-        poetry_env = self._get_poetry_virtualenv()
+        poetry_env = await self._get_poetry_virtualenv()
 
         if poetry_env:
             from ..virtualenv import Virtualenv
 
-            # Execute the task in the virtualenv from poetry
+            # Execute the task in the virtualenv from poetry, this is much faster than
+            # invoking `poetry run` each time.
             venv = Virtualenv(Path(poetry_env))
-            return self._execute_cmd(
+            return await self._execute_cmd(
                 (venv.resolve_executable(cmd[0]), *cmd[1:]),
                 input=input,
                 env=venv.get_env_vars(self.env.to_dict()),
@@ -47,25 +53,26 @@ class PoetryExecutor(PoeExecutor):
 
         if self._virtualenv_creation_disabled():
             # There's no poetry env, and there isn't going to be
-            return self._execute_cmd(cmd, input=input, use_exec=use_exec)
+            cmd = (*self._resolve_executable(cmd[0]), *cmd[1:])
+            return await self._execute_cmd(cmd, input=input, use_exec=use_exec)
 
         # Run this task with `poetry run`
-        return self._execute_cmd(
-            (self._poetry_cmd(), "run", *cmd),
+        return await self._execute_cmd(
+            (self._poetry_cmd(), "--no-plugins", "run", *cmd),
             input=input,
             use_exec=use_exec,
         )
 
-    def _handle_file_not_found(
+    async def _handle_file_not_found(
         self, cmd: Sequence[str], error: FileNotFoundError
-    ) -> int:
-        poetry_env = self._get_poetry_virtualenv()
+    ):
+        poetry_env = await self._get_poetry_virtualenv()
         error_context = f" using virtualenv {poetry_env!r}" if poetry_env else ""
-        raise PoeException(
+        raise ExecutionError(
             f"executable {cmd[0]!r} could not be found{error_context}"
         ) from error
 
-    def _get_poetry_virtualenv(self, force: bool = True):
+    async def _get_poetry_virtualenv(self):
         """
         Ask poetry where it put the virtualenv for this project.
         Invoking poetry is relatively expensive so cache the result
@@ -76,8 +83,8 @@ class PoetryExecutor(PoeExecutor):
 
         exec_cache = self.context.exec_cache
 
-        if force and "poetry_virtualenv" not in exec_cache:
-            from subprocess import PIPE, Popen
+        if "poetry_virtualenv" not in exec_cache:
+            from subprocess import PIPE
 
             # Need to make sure poetry isn't influenced by whatever virtualenv is
             # currently active
@@ -85,25 +92,25 @@ class PoetryExecutor(PoeExecutor):
             clean_env.pop("VIRTUAL_ENV", None)
             clean_env["PYTHONIOENCODING"] = "utf-8"
 
-            exec_cache["poetry_virtualenv"] = (
-                Popen(
-                    (self._poetry_cmd(), "env", "info", "-p"),
-                    stdout=PIPE,
-                    cwd=self.context.config.project_dir,
-                    env=clean_env,
-                )
-                .communicate()[0]
-                .decode()
-                .strip()
+            proc = await asyncio.create_subprocess_exec(
+                self._poetry_cmd(),
+                "--no-plugins",
+                "env",
+                "info",
+                "-p",
+                stdout=PIPE,
+                cwd=self.context.config.project_dir,
+                env=clean_env,
             )
+            outputs = await proc.communicate()
+            exec_cache["poetry_virtualenv"] = outputs[0].decode().strip()
 
         return exec_cache.get("poetry_virtualenv")
 
     @classmethod
     def _poetry_cmd(cls):
-        from_path = cls._poetry_cmd_from_path()
-        if from_path:
-            return str(Path(from_path).resolve())
+        if from_path := cls._poetry_cmd_from_path():
+            return from_path
 
         return "poetry"
 
